<?php

/*
 * Plugin Name: FEN Reader
 * Plugin URI: http://www.zenpawn.com/fen_reader/index.html
 * Description: Displays a chess position using Forsyth-Edwards Notation (FEN)
 * Version: 1.0
 * Author: Erin Dame
 * Author URI: http://www.zenpawn.com/fen_reader/index.html
 */

/*
 * Displays a chess position using Forsyth-Edwards Notation
 * (FEN), part of the Portable Game Notation (PGN) standard,
 * embedded within the special HTML tags <fen> and </fen>
 */

/*
 * FEN standard - from Wikipedia at
 * http://en.wikipedia.org/wiki/Forsyth-Edwards_Notation
 *
 * A FEN record contains 6 fields. The separator between fields is a space.
 * The fields are:
 *
 * 1. Piece placement (from white's perspective). Each rank is described, 
 * starting with rank 8 and ending with rank 1; within each rank, the contents 
 * of each square are described from file a through file h. White pieces are 
 * designated using upper-case letters ("KQRBNP"), Black by lowercase
 * ("kqrbnp"). Blank squares are noted using digits 1 through 8 (the number
 * of blank squares), and "/" separate ranks. 
 *
 * 2. Active color. "w" means white moves next, "b" means black. 
 *
 * 3. Castling availability. If neither side can castle, this is "-". Otherwise,  
 * this has one or more letters: "K" (White can castle kingside), "Q" (White can 
 * castle queenside), "k" (Black can castle kingside), and/or "q" (Black can 
 * castle queenside).
 *
 * 4. En passant target square in algebraic notation. If there's no en passant 
 * target square, this is "-". If a pawn has just made a 2-square move, this
 * is the position "behind" the pawn. 
 *
 * 5. Halfmove clock: This is the number of halfmoves since the last pawn 
 * advance or capture. This is used to determine if a draw can be claimed
 * under the fifty move rule. 
 *
 * 6. Fullmove number: The number of the full move. It starts at 1, and is 
 * incremented after Black's move.
 */

// a class is used to avoid function name collision
class fen_reader
{
	// hook into Options menu
	function admin()
	{
		if( function_exists('add_options_page') )
			add_options_page( 'FEN Reader', 'FEN Reader', 'manage_options', __FILE__, 'fen_reader_admin' );
	}

	// hook into the_content with <fen> </fen> tags
	function parse( $text )
	{
		return preg_replace_callback( '/<fen>((.|\n|\r)*?)<\/fen>/', 
array('fen_reader','render_position'), $text );
    	}

	// starting position
	function new_game() 
	{
		return "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";
	}

	// creates html for the square/piece image
	function square( $image_url, $piece, $square_color = 1 )
	{  
		return 	'<img src="' .
			$image_url .
			$piece .
			($square_color == 0 ? 'd' : 'l') .
			'.png" width=33 height=33 />';
	}

	// creates html for the diagram and position information
	function render_position( $fen )
	{  
		$image_url = get_option( 'fen_reader_images' );
		$square_color = 1;

		// remove <fen> </fen> tags
		$fen = strip_tags( $fen[0] );

		// trim leading whitespace
		$fen = ltrim( $fen );

		// at least draw the starting position
		if( $fen == '' )
			$fen = fen_reader::new_game();
	
		// parse diagram portion of string and construct html
		$html = '<center><table border=1><td>';
		for( $i = 0; $i < strlen($fen); $i++ )
		{
			// end of diagram
			if( ereg("[^KkQqBbNnRrPp0-9/]", $fen[$i]) )
				break;

			// end of rank
			if( $fen[$i] == '/' )
			{
				$html .= '<br>';
				$square_color = !$square_color;
				continue;
			}

			// empty squares
			if( ereg("[0-9]", $fen[$i]) )
			{
				for( $j = 0; $j < $fen[$i]; $j++ )
				{
					$html .= fen_reader::square( $image_url, '', $square_color );
					$square_color = !$square_color;
				}
				continue;
			}

			// a piece
			switch( $fen[$i] )
			{         
				case 'K': $piece = 'wk'; break;
				case 'Q': $piece = 'wq'; break;
				case 'R': $piece = 'wr'; break;
				case 'B': $piece = 'wb'; break;
				case 'N': $piece = 'wn'; break;
				case 'P': $piece = 'wp'; break;
				case 'k': $piece = 'bk'; break;
				case 'q': $piece = 'bq'; break;
				case 'r': $piece = 'br'; break;
				case 'b': $piece = 'bb'; break;    
				case 'n': $piece = 'bn'; break;
				case 'p': $piece = 'bp'; break;
			}
			$html .= fen_reader::square( $image_url, $piece, $square_color );

			// toggle square color
			$square_color = !$square_color;  
		}
		$html .= '</td></table>';
	
		// handle additional fields
		for( $field = 2; $i < strlen($fen); $i++ )
		{
			if( ereg("[^a-zA-Z0-9-]", $fen[$i]) )
				continue;

			switch( $field )
			{
				case 2:  // active color
					if( $fen[$i] == 'w' || $fen[$i] == 'W' )
						$html .= '<strong>White To Move</strong>';
					else if( $fen[$i] == 'b' || $fen[$i] == 'B' )
						$html .= '<strong>Black To Move</strong>';
					break;

				case 3:  // castling availability
					$done = FALSE;
					$neither = 0; $wk = 0; $wq = 0; $bk = 0; $bq = 0;
					while( $i < strlen($fen) && !$done )
					{
						switch( $fen[$i++] )
						{
							case '-': $neither = 1; break;
							case 'K': $wk = 1; break;
							case 'k': $bk = 1; break;
							case 'Q': $wq = 1; break;
							case 'q': $bq = 1; break;
							default: $done = TRUE; $i--; break;
						}
					}

					$html .= '<br>';

					if( $neither )
					{
						$html .= 'Neither side can castle.';
						break;
					}

					if( $wk )
					{
						$html .= 'White can castle kingside';
						if( $wq )
							$html .= ' or queenside';
						$html .= '.';
					}
					else if( $wq )
						$html .= 'White can castle queenside.';

					if( $wk || $wq )
						$html .= '<br>';

					if( $bk )
					{
						$html .= 'Black can castle kingside';
						if( $bq )
							$html .= ' or queenside';
						$html .= '.';
					}
					else if( $bq )
						$html .= 'Black can castle queenside.';

					break;

				case 4:  // en passant target square
					if( $fen[$i] == '-' )
						$html .= '<br>There are no en passant captures possible.';
					else
						$html .= '<br>There is an en passant possible at ' . $fen[$i] . $fen[++$i] . '.';
					break;

				case 5:  // halfmove clock
					$html .= '<br>The last capture or pawn advance was ';
					while( ereg("[0-9]", $fen[$i]) )
						$html .= $fen[$i++];
					$html .= ' half-moves ago.';
					break;

				case 6:  // fullmove number
					$html .= '<br>Position at move ';
					while( ereg("[0-9]", $fen[$i]) )
						$html .= $fen[$i++];
					$html .= '.';
					break;
			}

			$field++;
		}
		$html .= '</center>';

		return $html;
	}
}

// option administration panel
function fen_reader_admin()
{
	$location = get_option('siteurl') . '/wp-admin/admin.php?page=fen-reader.php'; // Form Action URI

 	if( isset($_POST['image_url']) )
	{
		if( !current_user_can('manage_options') )
			die(__('permission denied') );
		$image_url = $_POST['image_url'];
		update_option( 'fen_reader_images', $image_url );
	}
 
	echo '<div class="wrap">';
	echo '<h2>FEN Reader Configuration</h2>';
	echo '<form method="post" action="' . $location . '&amp;updated=true">';
	echo '<table width="100%" cellspacing="2" cellpadding="5" class="editform">';
	echo '<tr valign="top">';
	echo '<th scope="row">Image URL:</th>';
	echo '<td><input name="image_url" type="text" id="image_url" value=' . get_option('fen_reader_images') . ' size="80"/>';
	echo '<br />';
	echo 'Enter the URL to your chess images with a trailing forward-slash';
	echo '</tr>';
	echo '</table>';
	echo '<p class="submit">';
	echo '<input type="submit" name="submit" value="Update Options"/>';
	echo '</form>';
	echo '</div>';
}

add_action( 'admin_menu', array('fen_reader','admin') );
add_filter( 'the_content', array('fen_reader','parse') );
add_option( 'fen_reader_images', get_option('siteurl') );

?>
